// Based on code from ogl2brick-2.0 at
// http://developer.3dlabs.com/downloads/glslexamples/index.htm

#include <fcntl.h>
#if defined(OSX) || defined(Linux)
	#include <unistd.h>
#else
	#include <io.h>
#endif

int shaderSize(char *fileName)
{
    //
    // Returns the size in bytes of the shader fileName.
    // If an error occurred, it returns -1.
    //
    int fd;
    int count = -1;

    fd = open(fileName, O_RDONLY);
    if (fd != -1)
    {
        count = lseek(fd, 0, SEEK_END) + 1;
        close(fd);
    }

    return count;
}

int readShader(char *fileName,char *shaderText, int size)
{
    //
    // Reads a shader from the supplied file and returns the shader in the
    // arrays passed in. Returns 1 if successful, 0 if an error occurred.
    // The parameter size is an upper limit of the amount of bytes to read.
    // It is ok for it to be too big.
    //
    FILE *fh;
    int count;


    // Open the file
    fh = fopen(fileName, "r");
    if (!fh)
        return -1;

    // Get the shader from a file.
    fseek(fh, 0, SEEK_SET);
    count = (int) fread(shaderText, 1, size, fh);
    shaderText[count] = '\0';

    if (ferror(fh))
        count = 0;

    fclose(fh);
    return count;
}



int readShaderSource(char *fileName, GLchar **Shader)
{
    int shSize;

    //
    // Allocate memory to hold the source of our shaders.
    //
    shSize = shaderSize(fileName);

    if (shSize == -1)
    {
        printf("Cannot determine size of the shader %s\n", fileName);
        return 0;
    }

    *Shader = (GLchar *) malloc(shSize);

    // Read the source code
    if (!readShader(fileName,*Shader, shSize))
    {
        printf("Cannot read the file %s\n", fileName);
        return 0;
    }

    return 1;
}

// Courtesy of Andy Eyster
void printOGLShaderLog(GLuint shader)
{
	GLint log_length;
	char* log;

	// Get error log length
	glGetShaderiv(shader,GL_INFO_LOG_LENGTH,&log_length);

	// Dynamically allocate log buffer
	log = new char[log_length];
	
	// Get error log
	glGetShaderInfoLog(shader,log_length,&log_length,log);

	printf("%s\n",log);
	delete log;
}

// Courtesy of Andy Eyster
void printOGLProgramLog(GLuint program)
{
	GLint log_length;
	char* log;

	// Get error log length
	glGetProgramiv(program,GL_INFO_LOG_LENGTH,&log_length);

	// Dynamically allocate log buffer
	log = new char[log_length];
	
	// Get error log
	glGetProgramInfoLog(program,log_length,&log_length,log);

	printf("%s\n",log);
	delete log;
}

GLuint load_shaders(GLchar* vertexFile, GLchar* fragmentFile)
{
	// Shader source strings
	GLchar* vertexSrc;
	GLchar* fragmentSrc;
	GLuint vertexObj, fragObj, shaderProg;

	// GLSL flags
	GLint compiled, linked;

	// TODO: Read shader source from files

#ifndef OSX
	// Check for shader capabilities
	if (!glewIsSupported("GL_ARB_vertex_shader"))
	{
		printf("Programmable vertex shaders are not supported.\n");
		exit(0);
	}
	else if (!glewIsSupported("GL_ARB_fragment_shader"))
	{
		printf("Programmable fragment shaders are not supported.\n");
		exit(0);
	}
#endif

	// TODO: Create shader objects

	// TODO: Associate source with shader objects

	// TODO: Compile vertex shader

	// TODO: Compile fragment shader

	// TODO: Create shader program object

	// TODO: Attach shader objects to program

	// TODO: Link shaders

	return shaderProg;
}